@@ -423,7 +423,7 @@ gst_child_proxy_child_added (GstObject * object, GstObject * child)
 /**
  * gst_child_proxy_child_removed:
  * @object: the parent object
- * @child: the newly added child
+ * @child: the removed child
  *
  * Emits the "child-removed" signal.
  */
@@ -449,6 +449,8 @@ gst_child_proxy_base_init (gpointer g_class)
 	 *
 	 * Will be emitted after the @object was added to the @child_proxy.
 	 */
+    /* FIXME 0.11: use GST_TYPE_OBJECT as GstChildProxy only
+     * supports GstObjects */
     signals[CHILD_ADDED] =
         g_signal_new ("child-added", G_TYPE_FROM_CLASS (g_class),
         G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (GstChildProxyInterface,
@@ -462,6 +464,8 @@ gst_child_proxy_base_init (gpointer g_class)
 	 *
 	 * Will be emitted after the @object was removed from the @child_proxy.
 	 */
+    /* FIXME 0.11: use GST_TYPE_OBJECT as GstChildProxy only
+     * supports GstObjects */
     signals[CHILD_REMOVED] =
         g_signal_new ("child-removed", G_TYPE_FROM_CLASS (g_class),
         G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (GstChildProxyInterface,
@@ -475,9 +479,10 @@ gst_child_proxy_base_init (gpointer g_class)
 GType
 gst_child_proxy_get_type (void)
 {
-  static GType type = 0;
+  static volatile gsize type = 0;
 
-  if (G_UNLIKELY (type == 0)) {
+  if (g_once_init_enter (&type)) {
+    GType _type;
     static const GTypeInfo info = {
       sizeof (GstChildProxyInterface),
       gst_child_proxy_base_init,        /* base_init */
@@ -489,9 +494,12 @@ gst_child_proxy_get_type (void)
       0,                        /* n_preallocs */
       NULL                      /* instance_init */
     };
-    type = g_type_register_static (G_TYPE_INTERFACE, "GstChildProxy", &info, 0);
 
-    g_type_interface_add_prerequisite (type, GST_TYPE_OBJECT);
+    _type =
+        g_type_register_static (G_TYPE_INTERFACE, "GstChildProxy", &info, 0);
+
+    g_type_interface_add_prerequisite (_type, GST_TYPE_OBJECT);
+    g_once_init_leave (&type, (gsize) _type);
   }
   return type;
 }
