@@ -16,6 +16,12 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:gstmacros
+ * @short_description: Various portabillity helper macros
+ *
+ * A set of macros complementing the glib portability macros.
+ */
 
 #ifndef __GST_MACROS_H__
 #define __GST_MACROS_H__
@@ -24,7 +30,7 @@ G_BEGIN_DECLS
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
 # define GST_GNUC_CONSTRUCTOR \
-  __attribute__ ((constructor))  
+  __attribute__ ((constructor))
 #else  /* !__GNUC__ */
 # define GST_GNUC_CONSTRUCTOR
 #endif /* !__GNUC__ */
@@ -32,8 +38,11 @@ G_BEGIN_DECLS
 #if defined (__GNUC__) && !defined (GST_IMPLEMENT_INLINES)
 # define GST_INLINE_FUNC extern __inline__
 # define GST_CAN_INLINE 1
+#elif defined(_MSC_VER)
+# define GST_INLINE_FUNC extern __inline
+# define GST_CAN_INLINE 1
 #else
-# define GST_INLINE_FUNC extern 
+# define GST_INLINE_FUNC extern
 # undef GST_CAN_INLINE
 #endif
 
