@@ -241,8 +241,10 @@ gboolean        gst_base_src_query_latency (GstBaseSrc *src, gboolean * live,
                                             GstClockTime * min_latency, 
 					    GstClockTime * max_latency);
 
-void		gst_base_src_set_do_timestamp (GstBaseSrc *src,
-					       gboolean timestamp);
+void		gst_base_src_set_blocksize    (GstBaseSrc *src, gulong blocksize);
+gulong          gst_base_src_get_blocksize    (GstBaseSrc *src);
+
+void		gst_base_src_set_do_timestamp (GstBaseSrc *src, gboolean timestamp);
 gboolean	gst_base_src_get_do_timestamp (GstBaseSrc *src);
 
 G_END_DECLS
