@@ -134,7 +134,7 @@ interpolate_none_get_##type##_value_array (GstInterpolationControlSource *self,
   \
   g_mutex_lock (self->lock); \
   for(i = 0; i < value_array->nbsamples; i++) { \
-    ret = _interpolate_none_get_##type (self, timestamp); \
+    ret = _interpolate_none_get_##type (self, ts); \
     if (!ret) { \
       g_mutex_unlock (self->lock); \
       return FALSE; \
@@ -207,7 +207,7 @@ interpolate_none_get_boolean_value_array (GstInterpolationControlSource * self,
 
   g_mutex_lock (self->lock);
   for (i = 0; i < value_array->nbsamples; i++) {
-    ret = _interpolate_none_get (self, timestamp);
+    ret = _interpolate_none_get (self, ts);
     if (!ret) {
       g_mutex_unlock (self->lock);
       return FALSE;
@@ -231,7 +231,7 @@ interpolate_none_get_enum_value_array (GstInterpolationControlSource * self,
 
   g_mutex_lock (self->lock);
   for (i = 0; i < value_array->nbsamples; i++) {
-    ret = _interpolate_none_get (self, timestamp);
+    ret = _interpolate_none_get (self, ts);
     if (!ret) {
       g_mutex_unlock (self->lock);
       return FALSE;
@@ -255,7 +255,7 @@ interpolate_none_get_string_value_array (GstInterpolationControlSource * self,
 
   g_mutex_lock (self->lock);
   for (i = 0; i < value_array->nbsamples; i++) {
-    ret = _interpolate_none_get (self, timestamp);
+    ret = _interpolate_none_get (self, ts);
     if (!ret) {
       g_mutex_unlock (self->lock);
       return FALSE;
@@ -350,7 +350,7 @@ interpolate_trigger_get_##type##_value_array (GstInterpolationControlSource *sel
   \
   g_mutex_lock (self->lock); \
   for(i = 0; i < value_array->nbsamples; i++) { \
-    ret = _interpolate_trigger_get_##type (self, timestamp); \
+    ret = _interpolate_trigger_get_##type (self, ts); \
     if (!ret) { \
       g_mutex_unlock (self->lock); \
       return FALSE; \
@@ -425,7 +425,7 @@ interpolate_trigger_get_boolean_value_array (GstInterpolationControlSource *
 
   g_mutex_lock (self->lock);
   for (i = 0; i < value_array->nbsamples; i++) {
-    ret = _interpolate_trigger_get (self, timestamp);
+    ret = _interpolate_trigger_get (self, ts);
     if (!ret) {
       g_mutex_unlock (self->lock);
       return FALSE;
@@ -449,7 +449,7 @@ interpolate_trigger_get_enum_value_array (GstInterpolationControlSource * self,
 
   g_mutex_lock (self->lock);
   for (i = 0; i < value_array->nbsamples; i++) {
-    ret = _interpolate_trigger_get (self, timestamp);
+    ret = _interpolate_trigger_get (self, ts);
     if (!ret) {
       g_mutex_unlock (self->lock);
       return FALSE;
@@ -473,7 +473,7 @@ interpolate_trigger_get_string_value_array (GstInterpolationControlSource *
 
   g_mutex_lock (self->lock);
   for (i = 0; i < value_array->nbsamples; i++) {
-    ret = _interpolate_trigger_get (self, timestamp);
+    ret = _interpolate_trigger_get (self, ts);
     if (!ret) {
       g_mutex_unlock (self->lock);
       return FALSE;
@@ -596,7 +596,7 @@ DEFINE_LINEAR_GET (long, TRUE, EMPTY);
 
 DEFINE_LINEAR_GET (ulong, TRUE, EMPTY);
 DEFINE_LINEAR_GET (int64, TRUE, EMPTY);
-DEFINE_LINEAR_GET (uint64, TRUE, gst_util_guint64_to_gdouble);
+DEFINE_LINEAR_GET (uint64, TRUE, gst_guint64_to_gdouble);
 DEFINE_LINEAR_GET (float, FALSE, EMPTY);
 DEFINE_LINEAR_GET (double, FALSE, EMPTY);
 
@@ -672,7 +672,7 @@ _interpolate_cubic_update_cache_##vtype (GstInterpolationControlSource *self) \
   cp = node->data; \
   x_next = cp->timestamp; \
   y_next = g_value_get_##vtype (&cp->value); \
-  h[0] = gst_util_guint64_to_gdouble (x_next - x); \
+  h[0] = gst_guint64_to_gdouble (x_next - x); \
   \
   for (i = 1; i < n-1; i++) { \
     /* Shuffle x and y values */ \
@@ -685,7 +685,7 @@ _interpolate_cubic_update_cache_##vtype (GstInterpolationControlSource *self) \
     x_next = cp->timestamp; \
     y_next = g_value_get_##vtype (&cp->value); \
     \
-    h[i] = gst_util_guint64_to_gdouble (x_next - x); \
+    h[i] = gst_guint64_to_gdouble (x_next - x); \
     o[i] = h[i-1]; \
     p[i] = 2.0 * (h[i-1] + h[i]); \
     q[i] = h[i]; \
@@ -820,7 +820,7 @@ DEFINE_CUBIC_GET (long, TRUE, EMPTY);
 
 DEFINE_CUBIC_GET (ulong, TRUE, EMPTY);
 DEFINE_CUBIC_GET (int64, TRUE, EMPTY);
-DEFINE_CUBIC_GET (uint64, TRUE, gst_util_guint64_to_gdouble);
+DEFINE_CUBIC_GET (uint64, TRUE, gst_guint64_to_gdouble);
 DEFINE_CUBIC_GET (float, FALSE, EMPTY);
 DEFINE_CUBIC_GET (double, FALSE, EMPTY);
 
