@@ -62,9 +62,12 @@ struct _GstFdSrc {
   gint fd;
   gboolean seekable_fd;
 
+  /* poll timeout */
+  guint64 timeout;
+
   gchar *uri;
 
-  gint control_sock[2];
+  GstPoll *fdset;
 
   gulong curoffset; /* current offset in file */
 };
