@@ -343,6 +343,46 @@ GST_START_TEST (test_set_non_utf8_string)
 
 GST_END_TEST;
 
+GST_START_TEST (test_buffer_tags)
+{
+  GstTagList *tags;
+  GstBuffer *buf1, *buf2;
+
+  tags = gst_tag_list_new ();
+  buf1 = gst_buffer_new_and_alloc (222);
+  buf2 = gst_buffer_new_and_alloc (100);
+  gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_IMAGE, buf1,
+      GST_TAG_PREVIEW_IMAGE, buf2, NULL);
+  gst_buffer_unref (buf1);
+  gst_buffer_unref (buf2);
+
+  buf1 = buf2 = NULL;
+  fail_if (!gst_tag_list_get_buffer (tags, GST_TAG_IMAGE, &buf1));
+  gst_buffer_unref (buf1);
+  fail_if (!gst_tag_list_get_buffer (tags, GST_TAG_PREVIEW_IMAGE, &buf2));
+  gst_buffer_unref (buf2);
+
+  fail_if (gst_tag_list_get_buffer_index (tags, GST_TAG_IMAGE, 1, &buf1));
+  fail_if (gst_tag_list_get_buffer_index (tags, GST_TAG_IMAGE, 2, &buf1));
+  fail_if (gst_tag_list_get_buffer_index (tags, GST_TAG_PREVIEW_IMAGE, 1,
+          &buf1));
+  fail_if (gst_tag_list_get_buffer_index (tags, GST_TAG_PREVIEW_IMAGE, 2,
+          &buf1));
+
+  fail_if (!gst_tag_list_get_buffer_index (tags, GST_TAG_IMAGE, 0, &buf1));
+  fail_if (!gst_tag_list_get_buffer_index (tags, GST_TAG_PREVIEW_IMAGE, 0,
+          &buf2));
+  fail_unless_equals_int (GST_BUFFER_SIZE (buf1), 222);
+  fail_unless_equals_int (GST_BUFFER_SIZE (buf2), 100);
+
+  gst_buffer_unref (buf1);
+  gst_buffer_unref (buf2);
+
+  gst_tag_list_free (tags);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_tag_suite (void)
 {
@@ -356,6 +396,7 @@ gst_tag_suite (void)
   tcase_add_test (tc_chain, test_date_tags);
   tcase_add_test (tc_chain, test_type);
   tcase_add_test (tc_chain, test_set_non_utf8_string);
+  tcase_add_test (tc_chain, test_buffer_tags);
 
   return s;
 }
